import os
import sys

# 添加包的顶层目录
top_path = os.path.abspath(__file__)
top_path = top_path.split('jhsp')[0]
top_path = os.path.join(top_path, 'jhsp')
sys.path.append(top_path)

class GetXY():
    """

    """
    def __init__(self):
        pass

    def GetCrossXY(self,x,y,k,save=False):

        save_path = 'CrossValXY.pkl'

        xy_cross_val_list = []

        random_list = [i for i in range(x.shape[0])]
        np.random.shuffle(random_list)

        cross_num = x.shape[0] // k


        for time in range(k):
            if time == k-1:
                xy_cross_val_list.append((x.iloc[random_list, :],
                                          y.iloc[random_list[0:cross_num],:])
                                         )
                break

            xy_cross_val_list.append((x.iloc[random_list[0:cross_num],:],
                                      y.iloc[random_list[0:cross_num],:])
                                     )
            del random_list[0:cross_num]

        if save:
            with open(save_path,'wb') as f:
                pickle.dump(xy_cross_val_list,f)

        return xy_cross_val_list

    @staticmethod
    def LoadCrossXY(path):
        with open(path,'wb') as f:
            xy_cross_val_list = pickle.load(f)
            return xy_cross_val_list













import tensorflow as tf


class Model(tf.keras.Model):
    def __init__(self,initial_weigh,final_activation):
        super().__init__()

        self.layer_list = []

        for i,weights in enumerate(initial_weigh):

            if i != len(initial_weigh) -1:   #不是最后一层的时候，激活函数为rule
                exec("self.layer{} = tf.keras.layers.Dense(units=weights.shape[1],activation='relu' )".format(i))
            else:
                exec("self.layer{} = tf.keras.layers.Dense(units=weights.shape[1],activation='{}')".format(i,final_activation))

            exec('self.layer_list.append(self.layer{})'.format(i))

    def call(self, x):

        for layer in  self.layer_list:
            x = layer(x)


        return x






import tensorflow as tf

import sys
import os

# 添加包的顶层目录
top_path = os.path.abspath(__file__)
top_path = top_path.split('jhsp')[0]
top_path = os.path.join(top_path, 'jhsp')
sys.path.append(top_path)

from Sann.models import sann,ann,sanntuner
from Sann import getweights,weights22netweights




class GetSannModel():
    """
    返回SANN模型和网络结构一样的ANN模型
    """
    def __init__(self,x,y,how):

        gw = getweights.GetWeights(x, y)
        self.weights = gw.GetW()


        if how == 'mull':
            self.final_activation = 'sigmoid'
            self.loss = tf.keras.losses.BinaryCrossentropy()
            self.metrics = 'accuracy'
        if how == 'mulc':
            self.final_activation = 'softmax'
            self.loss = 'categorical_crossentropy'
            self.metrics = 'accuracy'

    # 标签还是分类 ，加一个参数
    def GetSann(self,hp):

        model = sanntuner.Model(self.weights, self.final_activation,hp)
        model.compile(loss=self.loss, optimizer='adam', metrics=[self.metrics])

        return model

    def GetCorAnn(self,x, y, hidden_layers):
        """
        获得与SANN网络结构一致的ANN
        :return:
        """
        gw = getweights.GetWeights(x, y)
        w22n = weights22netweights.W22NW()
        initial_weigh,error = w22n.Weights2HW(gw.GetW(), hidden_layers, False)

        model = ann.Model(initial_weigh, self.final_activation)
        model.compile(loss=self.loss, optimizer='adam', metrics=[self.metrics])

        return model









import os
import sys

# 添加包的顶层目录
top_path = os.path.abspath(__file__)
top_path = top_path.split('jhsp')[0]
top_path = os.path.join(top_path, 'jhsp')
sys.path.append(top_path)

class Sann:

    def __init__(self):
        from Sann import getweights,get_model,weights22netweights
        self.GetWeights = getweights.GetWeights
        self.GetSannModel = get_model.GetSannModel
        self.W22NW = weights22netweights.W22NW

class Model:

    def __init__(self):
        from Model.trad_models_parameters import GetTraMandP

        class Compare:
            def __init__(self):
                from Model.Compare import get_model_score
                self.GetModelScores = get_model_score.GetModelScores

        class Optimize:
            def __init__(self):
                from Model.Optimize import get_best_model
                self.GetBestModel = get_best_model.GetBestModel

        self.Compare = Compare()
        self.GetTraMandP = GetTraMandP
        self.Optimize = Optimize()

class FunTools:
    def __init__(self):
        from FunTools import add_top_path,my_pow,my_split
        self.add_top_path = add_top_path
        self.my_pow = my_pow
        self.my_split = my_split

class ClassTools:
    def __init__(self):
        from ClassTools import collatingwords
        self.CollatingWords = collatingwords.CollatingWords

class Data:
    def __init__(self):

        class GetXY:
            def __init__(self):
                from Data.GetXY import get_xy
                self.GetXY = get_xy.GetXY

        class Preprocessing:
            def __init__(self):
                from Data.Preprocessing import codingdata,describingdata,selectingfeatures
                self.CodingData = codingdata.CodingData
                self.DescribingData =describingdata.DescribingData
                self.SelectingFeatures = selectingfeatures.SelectingFeatures

        self.GetXY = GetXY()
        self.Preprocessing = Preprocessing()














